@@ -9,8 +9,9 @@ use DBIx::DBO::Table;
 use DBIx::DBO::Query;
 use DBIx::DBO::Row;
 
+our @ISA;
 our $use_c3_mro;
-my $need_c3_initialize;
+our $need_c3_initialize;
 
 BEGIN {
     # The C3 method resolution order is needed for optimal functioning when DBIx::DBO is being subclassed.
@@ -23,42 +24,42 @@ DBIx::DBO - An OO interface to SQL queries and results.  Easily constructs SQL q
 
 =cut
 
-our $VERSION = '0.03_03';
+our $VERSION = '0.04_01';
 
 =head1 SYNOPSIS
 
   use DBIx::DBO;
-
+  
   # Create the DBO
   my $dbo = DBIx::DBO->connect('DBI:mysql:my_db', 'me', 'mypasswd') or die $DBI::errstr;
-
-  # Create a "readonly" connection - useful for a slave database
+  
+  # Create a "read-only" connection - useful for a slave database
   $dbo->connect_readonly('DBI:mysql:my_db', 'me', 'mypasswd') or die $DBI::errstr;
-
+  
   # Start with a Query object
   my $query = $dbo->query('my_table');
-
+  
   # Find records with an 'o' in the name
   $query->where('name', 'LIKE', '%o%');
-
+  
   # And with an id that is less than 500
   $query->where('id', '<', 500);
-
+  
   # Exluding those with an age range from 20 to 29
   $query->where('age', 'NOT BETWEEN', [20, 29]);
-
+  
   # Return only the first 10 rows
   $query->limit(10);
-
+  
   # Fetch the rows
   while (my $row = $query->fetch) {
-
+  
       # Use the row as an array reference
       printf "id=%d  name=%s  status=%s\n", $row->[0], $row->[1], $row->[4];
-
+  
       # Or as a hash reference
       print 'id=', $row->{id}, "\n", 'name=', $row->{name};
-
+  
       # Update/delete rows
       $row->update(status => 'Fired!') if $row->{name} eq 'Harry';
       $row->delete if $record->{id} == 27;
@@ -66,13 +67,13 @@ our $VERSION = '0.03_03';
 
 =head1 DESCRIPTION
 
-This module provides a convenient and efficient way to access a database. It can construct queries for you and returns the results in easy to use methods.
+This module provides a convenient and efficient way to access a database.  It can construct queries for you and returns the results in easy to use methods.
 
-Once you've created a C<DBIx::DBO> object using one or both of C<connect> or C<connect_readonly>, you can begin creating C<DBIx::DBO::Query> objects. These are the "workhorse" objects, they encapsulate an entire query with JOINs, WHERE clauses, etc. You need not have to know about what created the C<Query> to be able to use or modify it. This makes it valuable in environments like mod_perl or large projects that prefer an object oreinted approach to data.
+Once you've created a C<DBIx::DBO> object using one or both of C<connect> or C<connect_readonly>, you can begin creating C<DBIx::DBO::Query> objects.  These are the "workhorse" objects, they encapsulate an entire query with JOINs, WHERE clauses, etc.  You need not have to know about what created the C<Query> to be able to use or modify it.  This makes it valuable in environments like mod_perl or large projects that prefer an object oreinted approach to data.
 
-The query is only automatically executed when the data is requested. This is to make it possible to minimise lookups that may not be needed or to delay them as late as possible.
+The query is only automatically executed when the data is requested.  This is to make it possible to minimise lookups that may not be needed or to delay them as late as possible.
 
-The C<DBIx::DBO::Row> object returned can be treated as both an arrayref or a hashref. The data is aliased for efficient use of memory. C<Row> objects can be updated or deleted, even when created by JOINs (If the DB supports it).
+The C<DBIx::DBO::Row> object returned can be treated as both an arrayref or a hashref.  The data is aliased for efficient use of memory.  C<Row> objects can be updated or deleted, even when created by JOINs (If the DB supports it).
 
 =head1 METHODS
 
@@ -93,16 +94,23 @@ sub import {
     }
 }
 
+=head3 C<new>
+
+  DBIx::DBO->new($dbh);
+  DBIx::DBO->new(undef, $readonly_dbh);
+
+Create a new C<DBIx::DBO> object from existsing C<DBI> handles.  You must provide one or both of the I<read-write> and I<read-only> C<DBI> handles.
+
 =head3 C<connect>
 
   $dbo = DBIx::DBO->connect($data_source, $username, $password, \%attr)
       or die $DBI::errstr;
 
-Takes the same arguments as L<DBI-E<gt>connect|DBI/"connect"> for a I<read-write> connection to a database. It returns the C<DBIx::DBO> object if the connection succeeds or undefined on failure.
+Takes the same arguments as L<DBI-E<gt>connect|DBI/"connect"> for a I<read-write> connection to a database.  It returns the C<DBIx::DBO> object if the connection succeeds or undefined on failure.
 
 =head3 C<connect_readonly>
 
-Takes the same arguments as C<connect> for a I<read-only> connection to a database. It returns the C<DBIx::DBO> object if the connection succeeds or undefined on failure.
+Takes the same arguments as C<connect> for a I<read-only> connection to a database.  It returns the C<DBIx::DBO> object if the connection succeeds or undefined on failure.
 
 Both C<connect> & C<connect_readonly> can be called on a C<DBIx::DBO> object to add that respective connection to create a C<DBIx::DBO> with both I<read-write> and I<read-only> connections.
 
@@ -113,23 +121,49 @@ Both C<connect> & C<connect_readonly> can be called on a C<DBIx::DBO> object to
 
 =cut
 
+sub new {
+    my $me = shift;
+    ouch 'Too many arguments for '.(caller(0))[3] if @_ > 3;
+    my $new;
+    if (@_ == 3 and defined($new = pop) and not UNIVERSAL::isa($new, 'HASH')) {
+        ouch '3rd argument to '.(caller(0))[3].' is not a HASH reference';
+    }
+    if (defined($new->{dbh} = shift)) {
+        ouch 'Invalid read-write database handle' unless blessed $new->{dbh} and $new->{dbh}->isa('DBI::db');
+        $new->{dbd} = $new->{dbh}{Driver}{Name};
+    }
+    if (defined($new->{rdbh} = shift)) {
+        ouch 'Invalid read-only database handle' unless blessed $new->{rdbh} and $new->{rdbh}->isa('DBI::db');
+        if ($new->{dbh}) {
+            ouch 'The read-write and read-only connections must use the same DBI driver'
+                if $new->{dbd} ne $new->{rdbh}{Driver}{Name};
+        } else {
+            $new->{dbd} = $new->{rdbh}{Driver}{Name};
+        }
+    }
+    ouch "Can't create the DBO, unknown database driver" unless $new->{dbd};
+
+    my $class = $me->_require_dbd_class($new->{dbd}) or return;
+    unless ($class) {
+        my $dbh = $new->{dbh} || $new->{rdbh};
+        $dbh->set_err('', $@) if $dbh;
+        return;
+    }
+    Class::C3::initialize() if $need_c3_initialize;
+    $class->_bless_dbo($new);
+}
+
 sub connect {
     my $me = shift;
     if (blessed $me) {
         ouch 'DBO is already connected' if $me->{dbh};
         $me->_check_driver($_[0]) if @_;
-        $me->{dbh} = $me->_connect($me->{ConnectArgs}, @_) or return;
+        $me->{dbh} = $me->_connect($me->{ConnectArgs} ||= [], @_) or return;
         return $me;
     }
-    my $new = { rdbh => undef, ConnectArgs => [], ConnectReadOnlyArgs => [] };
-    $new->{dbh} = $me->_connect($new->{ConnectArgs}, @_) or return;
-    my $class = $me->_require_dbd_class($new->{dbh}{Driver}{Name});
-    unless ($class) {
-        $new->{dbh}->set_err('', $@);
-        return;
-    }
-    Class::C3::initialize() if $need_c3_initialize;
-    $class->_bless_dbo($new);
+    my %new;
+    my $dbh = $me->_connect($new{ConnectArgs} = [], @_) or return;
+    $me->new($dbh, undef, \%new);
 }
 
 sub connect_readonly {
@@ -137,18 +171,12 @@ sub connect_readonly {
     if (blessed $me) {
         $me->{rdbh}->disconnect if $me->{rdbh};
         $me->_check_driver($_[0]) if @_;
-        $me->{rdbh} = $me->_connect($me->{ConnectReadOnlyArgs}, @_) or return;
+        $me->{rdbh} = $me->_connect($me->{ConnectReadOnlyArgs} ||= [], @_) or return;
         return $me;
     }
-    my $new = { dbh => undef, ConnectArgs => [], ConnectReadOnlyArgs => [] };
-    $new->{rdbh} = $me->_connect($new->{ConnectReadOnlyArgs}, @_) or return;
-    my $class = $me->_require_dbd_class($new->{rdbh}{Driver}{Name});
-    unless ($class) {
-        $new->{rdbh}->set_err('', $@);
-        return;
-    }
-    Class::C3::initialize() if $need_c3_initialize;
-    $class->_bless_dbo($new);
+    my %new;
+    my $dbh = $me->_connect($new{ConnectReadOnlyArgs} = [], @_) or return;
+    $me->new(undef, $dbh, \%new);
 }
 
 sub _check_driver {
@@ -158,6 +186,7 @@ sub _check_driver {
         ouch "Can't connect to data source '$dsn' because I can't work out what driver to use " .
             "(it doesn't seem to contain a 'dbi:driver:' prefix and the DBI_DRIVER env var is not set)";
     ref($me) =~ /::DBD::\Q$driver\E$/ or
+    $driver eq $me->{dbd} or
         ouch "Can't connect to the data source '$dsn'\n" .
             "The read-write and read-only connections must use the same DBI driver";
 }
@@ -171,7 +200,7 @@ sub _connect {
 
 ### Add a stack trace to PrintError & RaiseError
         $attr{HandleError} = sub {
-            if ($Config{_Debug_SQL} > 1) {
+            if ($Config{DebugSQL} > 1) {
                 $_[0] = Carp::longmess($_[0]);
                 return 0;
             }
@@ -197,7 +226,7 @@ sub _require_dbd_class {
     my @warn;
     {
         local $SIG{__WARN__} = sub { push @warn, join '', @_ };
-        return $me->_set_inheritance($dbd) if eval "require $class";
+        return $me->_set_dbd_inheritance($dbd) if eval "require $class";
     }
 
     (my $file = $class.'.pm') =~ s'::'/'g;
@@ -209,35 +238,31 @@ sub _require_dbd_class {
         die join "\n", "Can't load $dbd driver", @warn, $err;
     }
 
+    $@ = '';
     delete $INC{$file};
     $INC{$file} = 1;
-    return $me->_set_inheritance($dbd);
+    return $me->_set_dbd_inheritance($dbd);
 }
 
-sub _set_inheritance {
-    my $me = shift;
+sub _set_dbd_inheritance {
+    my $class = shift;
     my $dbd = shift;
-    my $class = $me.'::DBD::'.$dbd;
-
     no strict 'refs';
-    return $class if "@{$class.'::Common::ISA'}" eq $me.'::Common';
-
-    @{$class.'::Common::ISA'} = ($me.'::Common');
-    @{$class.'::ISA'} = ($me, $class.'::Common');
-    @{$class.'::'.$_.'::ISA'} = ($me.'::'.$_, $class.'::Common') for qw(Table Query Row);
-    if ($me ne __PACKAGE__) {
-        push @{$class.'::ISA'}, __PACKAGE__.'::DBD::'.$dbd;
-        push @{$class.'::'.$_.'::ISA'}, __PACKAGE__.'::DBD::'.$dbd.'::'.$_ for qw(Table Query Row);
-        eval "package ${me}";
-        eval "package ${me}::$_" for qw(Common Table Query Row);
+    my @isa = @_ ? @_ : @{$class.'::ISA'};
+    if ($class eq __PACKAGE__) {
+        # Let DBIx::DBO secretly inherit from DBIx::DBO::Common
+        push @isa, 'DBIx::DBO::Common' unless @_;
+    } else {
+        for my $obj (qw(Table Query Row)) {
+            unless (@{$class.'::'.$obj.'::ISA'}) {
+                @{$class.'::'.$obj.'::ISA'} = map UNIVERSAL::isa($_, __PACKAGE__) ? $_.'::'.$obj : $_, @isa;
+            }
+        }
     }
-    if ($use_c3_mro) {
-        mro::set_mro($class, 'c3');
-        mro::set_mro($class.'::'.$_, 'c3') for qw(Table Query Row);
-        # If perl < 5.9.5 then we need to call Class::C3::initialize()
-        $need_c3_initialize = $] < 5.009_005;
+    for my $class (map $class.'::'.$_, qw(Table Query Row)) {
+        $class->_set_dbd_inheritance($dbd);
     }
-    return $class;
+    $class->DBIx::DBO::Common::_set_dbd_inheritance($dbd, @isa);
 }
 
 sub _bless_dbo {
@@ -258,8 +283,8 @@ Tables can be specified by their name or an arrayref of schema and table name or
 =cut
 
 sub table {
-    my $class = ref($_[0]).'::Table';
-    $class->_new(@_);
+    (my $class = ref($_[0])) =~ s/(::DBD::\w+)$/::Table/;
+    $class->new(@_);
 }
 
 =head3 C<query>
@@ -277,8 +302,8 @@ In list context, the C<Query> object and L<DBIx::DBO::Table|DBIx::DBO::Table> ob
 =cut
 
 sub query {
-    my $class = ref($_[0]).'::Query';
-    $class->_new(@_);
+    (my $class = ref($_[0])) =~ s/(::DBD::\w+)$/::Query/;
+    $class->new(@_);
 }
 
 =head3 C<row>
@@ -291,27 +316,27 @@ Create and return a new L<DBIx::DBO::Row|DBIx::DBO::Row> object.
 =cut
 
 sub row {
-    my $class = ref($_[0]).'::Row';
-    $class->_new(@_);
+    (my $class = ref($_[0])) =~ s/(::DBD::\w+)$/::Row/;
+    $class->new(@_);
 }
 
 =head3 C<selectrow_array>
 
   $dbo->selectrow_array($statement, \%attr, @bind_values);
 
-This provides access to the L<DBI-E<gt>selectrow_array|DBI/"selectrow_array"> method.
+This provides access to the L<DBI-E<gt>selectrow_array|DBI/"selectrow_array"> method.  It defaults to using the I<read-only> C<DBI> handle.
 
 =head3 C<selectrow_arrayref>
 
   $dbo->selectrow_arrayref($statement, \%attr, @bind_values);
 
-This provides access to the L<DBI-E<gt>selectrow_arrayref|DBI/"selectrow_arrayref"> method.
+This provides access to the L<DBI-E<gt>selectrow_arrayref|DBI/"selectrow_arrayref"> method.  It defaults to using the I<read-only> C<DBI> handle.
 
 =head3 C<selectall_arrayref>
 
   $dbo->selectall_arrayref($statement, \%attr, @bind_values);
 
-This provides access to the L<DBI-E<gt>selectall_arrayref|DBI/"selectall_arrayref"> method.
+This provides access to the L<DBI-E<gt>selectall_arrayref|DBI/"selectall_arrayref"> method.  It defaults to using the I<read-only> C<DBI> handle.
 
 =cut
 
@@ -451,22 +476,36 @@ The I<read-only> C<DBI> handle, or if there is no I<read-only> connection, the I
   $dbo->do($statement, \%attr) or die $dbo->dbh->errstr;
   $dbo->do($statement, \%attr, @bind_values) or die ...
 
-This provides access to the L<DBI-E<gt>do|DBI/"do"> method. It defaults to using the I<read-write> C<DBI> handle.
+This provides access to the L<DBI-E<gt>do|DBI/"do"> method.  It defaults to using the I<read-write> C<DBI> handle.
 
 =cut
 
+sub _handle {
+    my $me = shift;
+    my $handle = shift;
+    my ($d, $c) = $handle ne 'read-only' ? qw(dbh ConnectArgs) : qw(rdbh ConnectReadOnlyArgs);
+    ouch "No $handle handle connected" unless defined $me->{$d};
+    # Automatically reconnect, but only if possible and needed
+    $me->{$d} = $me->_connect($me->{$c}) if exists $me->{$c} and not $me->{$d}->ping;
+    return $me->{$d};
+}
+
 sub dbh {
     my $me = shift;
+    if (my $handle = $me->config('UseHandle')) {
+        return $me->_handle($handle);
+    }
     ouch 'Invalid action for a read-only connection' unless $me->{dbh};
-    return $me->{dbh} if $me->{dbh}->ping;
-    $me->{dbh} = $me->_connect($me->{ConnectArgs});
+    $me->_handle('read-write');
 }
 
 sub rdbh {
     my $me = shift;
+    if (my $handle = $me->config('UseHandle')) {
+        return $me->_handle($handle);
+    }
     return $me->dbh unless $me->{rdbh};
-    return $me->{rdbh} if $me->{rdbh}->ping;
-    $me->{rdbh} = $me->_connect($me->{ConnectReadOnlyArgs});
+    $me->_handle('read-only');
 }
 
 =head3 C<config>
@@ -476,28 +515,31 @@ sub rdbh {
   $dbo_setting = $dbo->config($option);
   $dbo->config($option => $dbo_setting);
 
-Get or set the global or C<DBIx::DBO> config settings.
-When setting an option, the previous value is returned.
+Get or set the global or C<DBIx::DBO> config settings.  When setting an option, the previous value is returned.  When getting an option's value, if the value is undefined, the global value is returned.
 
-Options include:
+=head2 Available C<config> options
 
 =over
 
 =item C<QuoteIdentifier>
 
 Boolean setting to control quoting of SQL identifiers (schema, table and column names).
-Defaults to C<1>.
 
-=item C<_Debug_SQL>
+=item C<UseHandle>
 
-Set to C<1> or C<2> to warn about each SQL command executed. C<2> adds a full stack trace.
+Set to C<'read-write'> or C<'read-only'> to force using only that handle for all operations.
+Defaults to C<false> which chooses the I<read-only> handle for reads and the I<read-write> handle otherwise.
+
+=item C<DebugSQL>
+
+Set to C<1> or C<2> to warn about each SQL command executed.  C<2> adds a full stack trace.
 Defaults to C<0> (silent).
 
 =back
 
 Global options can also be set when C<use>'ing the module:
 
-  use DBIx::DBO QuoteIdentifier => 0, _Debug_SQL => 1;
+  use DBIx::DBO QuoteIdentifier => 0, DebugSQL => 1;
 
 =cut
 
@@ -505,13 +547,11 @@ sub config {
     my $me = shift;
     my $opt = shift;
     unless (blessed $me) {
-        my $val = $Config{$opt};
-        $Config{$opt} = shift if @_;
-        return $val;
+        return DBIx::DBO::Common->_set_config(\%Config, $opt, shift) if @_;
+        return $Config{$opt};
     }
-    my $val = defined $me->{Config}{$opt} ? $me->{Config}{$opt} : $Config{$opt};
-    $me->{Config}{$opt} = shift if @_;
-    return $val;
+    return $me->_set_config($me->{Config} ||= {}, $opt, shift) if @_;
+    return defined $me->{Config}{$opt} ? $me->{Config}{$opt} : $Config{$opt};
 }
 
 sub DESTROY {
@@ -532,17 +572,21 @@ You can find more information for this module at:
 
 =over 4
 
-=item * RT: CPAN's request tracker
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=DBIx-DBO>
+=item *
 
-=item * AnnoCPAN: Annotated CPAN documentation
-L<http://annocpan.org/dist/DBIx-DBO>
+RT: CPAN's request tracker L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=DBIx-DBO>
 
-=item * CPAN Ratings
-L<http://cpanratings.perl.org/d/DBIx-DBO>
+=item *
 
-=item * Search CPAN
-L<http://search.cpan.org/dist/DBIx-DBO>
+AnnoCPAN: Annotated CPAN documentation L<http://annocpan.org/dist/DBIx-DBO>
+
+=item *
+
+CPAN Ratings L<http://cpanratings.perl.org/d/DBIx-DBO>
+
+=item *
+
+Search CPAN L<http://search.cpan.org/dist/DBIx-DBO>
 
 =back
 
