@@ -6,6 +6,8 @@ use warnings;
 
 use overload '**' => \&column, fallback => 1;
 
+our @ISA;
+
 =head1 NAME
 
 DBIx::DBO::Table - An OO interface to SQL queries and results.  Encapsulates a table in an object.
@@ -14,34 +16,54 @@ DBIx::DBO::Table - An OO interface to SQL queries and results.  Encapsulates a t
 
   # Create a Table object
   my $table = $dbo->table('my_table');
-
+  
   # Get a column reference
   my $column = $table ** 'employee_id';
-
+  
   # Quickly display my employee id
   print $table->fetch_value('employee_id', name => 'Vernon');
-
+  
   # Insert a new row into the table
   $table->insert(employee_id => 007, name => 'James Bond');
-
+  
   # Remove rows from the table where the name IS NULL
   $table->delete(name => undef);
 
 =head1 METHODS
 
+=head3 C<new>
+
+  DBIx::DBO::Table->new($dbo, $table);
+  DBIx::DBO::Table->new($dbo, [$schema, $table]);
+  DBIx::DBO::Table->new($dbo, $table_object);
+
+Create and return a new C<Table> object.
+Tables can be specified by their name or an arrayref of schema and table name or another C<Table> object.
+
 =cut
 
-sub _new {
+sub new {
     my ($proto, $dbo, $table) = @_;
     my $class = ref($proto) || $proto;
     blessed $dbo and $dbo->isa('DBIx::DBO') or ouch 'Invalid DBO Object';
     (my $schema, $table, $_) = $dbo->table_info($table) or ouch 'No such table: '.$table;
+    $class = $class->_create_dbd_class($dbo->{dbd});
     bless { %$_, Schema => $schema, Name => $table, DBO => $dbo, LastInsertID => undef }, $class;
 }
 
+*_create_dbd_class = \&DBIx::DBO::Common::_create_dbd_class;
+
+sub _set_dbd_inheritance {
+    my $class = shift;
+    my $dbd = shift;
+    # Let DBIx::DBO::Table secretly inherit from DBIx::DBO::Common
+    @_ = (@ISA, 'DBIx::DBO::Common') if not @_ and $class eq __PACKAGE__;
+    $class->DBIx::DBO::Common::_set_dbd_inheritance($dbd, @_);
+}
+
 =head3 C<tables>
 
-Return a list of L<DBIx::DBO::Table|DBIx::DBO::Table> objects, which will always be this C<Table> object.
+Return a list of C<Table> objects, which will always be this C<Table> object.
 
 =cut
 
@@ -73,7 +95,7 @@ sub column {
     my ($me, $col) = @_;
     ouch 'Invalid column '.$me->_qi($col).' in table '.$me->_quoted_name
         unless exists $me->{Column_Idx}{$col};
-    defined $me->{Column}{$col} ? $me->{Column}{$col} : ($me->{Column}{$col} = bless [$me, $col], 'DBIx::DBO::Column');
+    $me->{Column}{$col} ||= bless [$me, $col], 'DBIx::DBO::Column';
 }
 
 =head3 C<fetch_row>
@@ -219,17 +241,17 @@ This provides access to L<DBI-E<gt>do|DBI/"do"> method.  It defaults to using th
   $table_setting = $table->config($option);
   $table->config($option => $table_setting);
 
-Get or set the C<Table> config settings.
-When setting an option, the previous value is returned.
+Get or set the C<Table> config settings.  When setting an option, the previous value is returned.  When getting an option's value, if the value is undefined, the L<DBIx::DBO|DBIx::DBO>'s value is returned.
+
+See L<DBIx::DBO/Available_config_options>.
 
 =cut
 
 sub config {
     my $me = shift;
     my $opt = shift;
-    my $val = defined $me->{Config}{$opt} ? $me->{Config}{$opt} : $me->{DBO}->config($opt);
-    $me->{Config}{$opt} = shift if @_;
-    return $val;
+    return $me->_set_config($me->{Config} ||= {}, $opt, shift) if @_;
+    return defined $me->{Config}{$opt} ? $me->{Config}{$opt} : $me->{DBO}->config($opt);
 }
 
 sub DESTROY {
@@ -240,6 +262,16 @@ sub DESTROY {
 
 __END__
 
+=head1 TODO LIST
+
+=over 4
+
+=item *
+
+Add a multi_insert method for extended INSERTs.
+
+=back
+
 =head1 SEE ALSO
 
 L<DBIx::DBO>
